// Package dot is a helper for producing graphs in DOT format (graphviz).
// It offers Graph,Node and Edge objects to construct simple and complex (e.g. nested) graphs.
//
// Copyright (c) Ernest Micklei. MIT License
package dot // import "github.com/emicklei/dot"
